#include "prehtml.h"

#ifdef _WIN32
#define NOMFINFICHIER "%TEMP%/prehtml_fingenerate_XXXXXX"
#else
#define NOMFINFICHIER "/tmp/prehtml_fingenerate_XXXXXX"
#endif // _WIN32
static char nomfinfichier[]=NOMFINFICHIER;

static vector<generate> generates;
int engenerate; /* 0=pas de générate, 1=dans générate
 * 2=/generate passé */

void init_generate(void){
	engenerate=0;
	generates.clear();
}

void remplitgenerate(const char *s){ /* partie commune à la 1ère fois et aux autres*/
	generate gene;
	gene.nom = s;
	gene.nomtemp = string(s)+".tmp";
	creetemp(gene.nomtemp.c_str()); /* fichier à effacer si arrêt d'urgence */
	sortie=fopen(gene.nomtemp.c_str(),"w");
	if (!sortie) erreur("<generate>: pas pu créer '%s'",gene.nomtemp.c_str());
	generates.push_back(gene);
}

void genere(const char *s){
	int i;
	generate *gene;
	if (!(*s)) erreur("<generate> avec une chaîne vide!");
	if (engenerate==1)erreur("Impossible de générer %s car on n'a pas"
	                         " refermé le <generate> précédent",s);
	else if (engenerate==0){ /* premier passage par <generate> */
		fclose(sortie); /* on ferme le fichier en cours, c'est le début de tous */
		engenerate=1;
		remplitgenerate(s);
	}
	else if (engenerate==2){ /* </generate> est passé, et on en fait un autre */
		/* d'abord, on vérifie que le nom donné n'a pas déjà été donné par un
		 * <generate> précedent */
		for (size_t n=0; n<generates.size(); n++){
			gene = &generates[n];
			if (!gene->nom.compare(s))
				erreur("<generate %s>: La commande <generate> n°%d voulait déjà "
				       "créer le fichier %s, or il est impossible de créer deux "
				       "fois le même fichier dans le même fichier .prehtml",
				       s,n,s);
		}
		/* on va vérifier que le fichier "suite" est bien vide
		 * Attention: il faut avoir été créé avec "w+" !*/
		fseek(sortie,0,SEEK_SET);
		while(1){
			i=getc(sortie);
			if (i==EOF) break;
			if (!strchr(" \t\n\r",i))
				erreur("<GENERATE %s> alors que des caractères non-blancs ('%c') "
				       "sont passés depuis le dernier </GENERATE>",s,i);
		}
		fclose(sortie); /* si c'est bon, on ferme et on efface le fichier de fin */
		unlink(nomfinfichier);
		remplitgenerate(s);
		engenerate=1;
	}
	else erreur("Erreur interne dans genere(): engenerate=%d",engenerate);
}

void degenere(void){ /* fonction appelée quand </generate> arrive */
	if (engenerate==0) erreur("</GENERATE> sans aucun <GENERATE> avant");
	else if (engenerate==2)
		erreur("</GENERATE> trouvé alors qu'il y en a déjà eu un avant");
	else if (engenerate==1){ /* OK, on est bien dans un <GENERATE>*/
		fclose(sortie);
		strcpy(nomfinfichier,NOMFINFICHIER);
	#ifdef _WIN32
		_mktemp(nomfinfichier);
	#else
		mkstemp(nomfinfichier);
	#endif // _WIN32
		creetemp(nomfinfichier); /* fichier à effacer si arrêt d'urgence */
		sortie=fopen(nomfinfichier,"w+"); /* le "+" est indispensable, on va le relire*/
		if (!sortie) erreur("</GENERATE>: pas pu créer le fichier de fin %s",
		                    nomfinfichier);
		engenerate=2;
	}
	else erreur("Erreur interne dans degenere(): engenerate=%d",engenerate);
}

void fin_generate(void){ /* choses à faire à la fin du fichier principal pour
	 * finaliser les <GENERATE> éventuels */
    char fichtempgene[]=
#ifdef _WIN32
    "%TEMP%/fichtempgenerateXXXXXX";
#else
    "/tmp/fichtempgenerateXXXXXX";
#endif

	FILE *fin,*fout,*f1,*f2;
	generate *gene;
	int i,i1,i2;

	if (engenerate==0) return; /* rien à faire */
	if (engenerate==1) erreur("EOF rencontré à l'intérieur d'un <GENERATE>");
	/* donc engenerate==2: on a des choses à faire */
	#ifdef _WIN32
	_mktemp(fichtempgene);
	#else
	mkstemp(fichtempgene);
	#endif // _WIN32
	creetemp(fichtempgene); /* fichier à effacer si arrêt d'urgence */
	for (size_t n=0; n<generates.size(); n++){ /* on parcourt tous les fichiers générés */
		gene = &generates[n];
		fout=fopen(fichtempgene,"wb");
		if (!fout)erreur("Finalisation des <generate>: impossible de créer %s",
		                 fichtempgene);
		fin=fopen(fich_temp.c_str(),"rb"); /* c'est finalement le fichier de début */
		if (!fin)erreur("Finalisation des <generate>: impossible de lire "
		                "le début dans %s",fich_temp.c_str());
		while ((i=fgetc(fin))!=EOF) putc(i,fout);
		fclose(fin);
		fin=fopen(gene->nomtemp.c_str(),"rb"); /* milieu */
		if (!fin)erreur("Finalisation des <generate>: impossible de lire "
		                "le milieu dans %s",gene->nomtemp.c_str());
		while ((i=fgetc(fin))!=EOF) putc(i,fout);
		fclose(fin);
		unlink(gene->nomtemp.c_str());
		fin=fopen(nomfinfichier,"rb"); /* fin */
		if (!fin)erreur("Finalisation des <generate>: impossible de lire "
		                "la fin dans %s",nomfinfichier);
		while ((i=fgetc(fin))!=EOF) putc(i,fout);
		fclose(fin);
		fclose(fout);
		/* on va comparer le fichier créé avec celui existant [s'il existe] */
		f1=fopen(gene->nom.c_str(),"rb");
		i1=0;i1=1;
		if (f1){
			f2=fopen(fichtempgene,"rb");
			if (!f2) erreur("Le fichier %s n'a pas été créé",fichtempgene);
			do{
				i1=getc(f1); i2=getc(f2);
			}while(i1==i2 && i1!=EOF);
			fclose(f2);fclose(f1);
		}
		if (i1!=i2 || forcewrite){
            if (mv(fichtempgene,gene->nom.c_str()))
                if (rename(fichtempgene,gene->nom.c_str()))
                    erreur("Fin GENERATE: Impossible de renommer %s en %s",
                        fichtempgene,gene->nom.c_str());
		}
		else unlink(fichtempgene); /* fichiers identiques: je l'enlève
		 * pour ne pas changer la date */
	}
	unlink(fich_temp.c_str()); /* n'a plus d'utilité, ce n'était que le début */
	unlink(nomfinfichier); /* n'a plus d'utilité, ce n'était que la fin */
}
